@@ -24,6 +24,8 @@
 #endif
 
 #include <gst/gl/gl.h>
+
+#include "gstglelements.h"
 #include "gstglcolorconvertelement.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_gl_color_convert_element_debug);
@@ -35,6 +37,8 @@ G_DEFINE_TYPE_WITH_CODE (GstGLColorConvertElement, gst_gl_color_convert_element,
     GST_DEBUG_CATEGORY_INIT (gst_gl_color_convert_element_debug,
         "glconvertelement", 0, "convert");
     );
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (glcolorconvert, "glcolorconvert",
+    GST_RANK_NONE, GST_TYPE_GL_COLOR_CONVERT_ELEMENT, gl_element_init (plugin));
 
 static gboolean gst_gl_color_convert_element_gl_set_caps (GstGLBaseFilter *
     base_filter, GstCaps * in_caps, GstCaps * out_caps);
@@ -149,7 +153,7 @@ gst_gl_color_convert_element_transform_caps (GstBaseTransform * bt,
   GstCaps *ret;
 
   if (base_filter->display && !gst_gl_base_filter_find_gl_context (base_filter))
-    return NULL;
+    return gst_caps_new_empty ();
 
   context = gst_gl_base_filter_get_gl_context (base_filter);
 
