@@ -44,7 +44,6 @@
 #include "config.h"
 #endif
 
-#include <sys/stat.h>
 #include "gstshmallocator.h"
 
 #ifdef HAVE_MMAP
@@ -54,6 +53,11 @@
 #include <sys/mman.h>
 #endif
 
+struct _GstShmAllocator
+{
+  GstFdAllocator parent;
+};
+
 #define GST_CAT_DEFAULT gst_shm_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
