@@ -18,14 +18,15 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef __GST_GL_EGL_H__
-#define __GST_GL_EGL_H__
+#ifndef __GST_GL_CONTEXT_EGL_H__
+#define __GST_GL_CONTEXT_EGL_H__
 
 #include <gst/gl/gstglcontext.h>
 #include <gst/gl/egl/gstgldisplay_egl.h>
 
 G_BEGIN_DECLS
 
+typedef struct _GstGLDmaModifier GstGLDmaModifier;
 typedef struct _GstGLContextEGL GstGLContextEGL;
 typedef struct _GstGLContextEGLClass GstGLContextEGLClass;
 
@@ -33,11 +34,25 @@ G_GNUC_INTERNAL GType gst_gl_context_egl_get_type (void);
 #define GST_TYPE_GL_CONTEXT_EGL         (gst_gl_context_egl_get_type())
 
 #define GST_GL_CONTEXT_EGL(o)           (G_TYPE_CHECK_INSTANCE_CAST((o), GST_TYPE_GL_CONTEXT_EGL, GstGLContextEGL))
-#define GST_GL_CONTEXT_EGL_CLASS(k)     (G_TYPE_CHECK_CLASS((k), GST_TYPE_GL_CONTEXT_EGL, GstGLContextEGLClass))
+#define GST_GL_CONTEXT_EGL_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), GST_TYPE_GL_CONTEXT_EGL, GstGLContextEGLClass))
 #define GST_IS_GL_CONTEXT_EGL(o)        (G_TYPE_CHECK_INSTANCE_TYPE((o), GST_TYPE_GL_CONTEXT_EGL))
 #define GST_IS_GL_CONTEXT_EGL_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE((k), GST_TYPE_GL_CONTEXT_EGL))
 #define GST_GL_CONTEXT_EGL_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS((o), GST_TYPE_GL_CONTEXT_EGL, GstGLContextEGLClass))
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstGLContextEGL, gst_object_unref)
+
+/**
+ * GstGLDmaModifier: (skip)
+ *
+ * Opaque struct
+ */
+struct _GstGLDmaModifier
+{
+  /*< private >*/
+  guint64 modifier;
+  gboolean external_only;
+};
+
 /**
  * GstGLContextEGL:
  *
@@ -45,7 +60,7 @@ G_GNUC_INTERNAL GType gst_gl_context_egl_get_type (void);
  */
 struct _GstGLContextEGL
 {
-  /* <private> */
+  /*< private >*/
   GstGLContext context;
 
   GstGLDisplayEGL *display_egl;
@@ -64,6 +79,11 @@ struct _GstGLContextEGL
 
   /* Cached handle */
   guintptr window_handle;
+  gulong window_handle_signal;
+
+  GstStructure *requested_config;
+
+  GArray *dma_formats;
 };
 
 /**
@@ -73,7 +93,7 @@ struct _GstGLContextEGL
  */
 struct _GstGLContextEGLClass
 {
-  /* <private> */
+  /*< private >*/
   GstGLContextClass parent;
 };
 
@@ -86,6 +106,15 @@ guintptr            gst_gl_context_egl_get_current_context  (void);
 G_GNUC_INTERNAL
 gpointer            gst_gl_context_egl_get_proc_address     (GstGLAPI gl_api, const gchar * name);
 
+G_GNUC_INTERNAL
+gboolean            gst_gl_context_egl_fill_info            (GstGLContext * context, GError ** error);
+
+G_GNUC_INTERNAL
+gboolean            gst_gl_context_egl_get_format_modifiers (GstGLContext * context,
+                                                             gint fourcc,
+                                                             const GArray ** modifiers);
+G_GNUC_INTERNAL
+gboolean            gst_gl_context_egl_supports_modifier     (GstGLContext * context);
 G_END_DECLS
 
-#endif /* __GST_GL_EGL_H__ */
+#endif /* __GST_GL_CONTEXT_EGL_H__ */
