@@ -79,7 +79,7 @@ static const gchar *schema_list[] = {
  *
  * Gets the list of supported schemas in the xmp lib
  *
- * Returns: (transfer none): a %NULL terminated array of strings with the
+ * Returns: (transfer none) (array zero-terminated=1): a %NULL terminated array of strings with the
  *     schema names
  */
 const gchar **
@@ -276,7 +276,7 @@ gst_xmp_tag_create (const gchar * gst_tag, const gchar * xmp_tag,
 {
   XmpTag *xmpinfo;
 
-  xmpinfo = g_slice_new (XmpTag);
+  xmpinfo = g_new (XmpTag, 1);
   xmpinfo->gst_tag = gst_tag;
   xmpinfo->tag_name = xmp_tag;
   xmpinfo->type = xmp_type;
@@ -485,7 +485,8 @@ deserialize_exif_gps_coordinate (XmpTag * xmptag, GstTagList * taglist,
 
   /* check if it uses ,SS or .mm */
   if (strchr (current, ',') != NULL) {
-    sscanf (current, "%d,%d%c", &m, &s, &c);
+    if (!sscanf (current, "%d,%d%c", &m, &s, &c))
+      goto error;
   } else {
     gchar *copy = g_strdup (current);
     gint len = strlen (copy);
@@ -652,7 +653,7 @@ deserialize_exif_altitude (XmpTag * xmptag, GstTagList * taglist,
 
   /* clean up entry */
   g_free (ptag->str);
-  g_slice_free (PendingXmpTag, ptag);
+  g_free (ptag);
   *pending_tags = g_slist_delete_link (*pending_tags, entry);
 }
 
@@ -747,7 +748,7 @@ deserialize_exif_gps_speed (XmpTag * xmptag, GstTagList * taglist,
 
   /* clean up entry */
   g_free (ptag->str);
-  g_slice_free (PendingXmpTag, ptag);
+  g_free (ptag);
   *pending_tags = g_slist_delete_link (*pending_tags, entry);
 }
 
@@ -810,11 +811,11 @@ deserialize_exif_gps_direction (XmpTag * xmptag, GstTagList * taglist,
   }
 
   if (!dir_str) {
-    GST_WARNING ("Missing %s tag", dir_str);
+    GST_WARNING ("Missing GPSImgDirection tag");
     return;
   }
   if (!dirref_str) {
-    GST_WARNING ("Missing %s tag", dirref_str);
+    GST_WARNING ("Missing GPSImgDirectionRef tag");
     return;
   }
 
@@ -841,7 +842,7 @@ deserialize_exif_gps_direction (XmpTag * xmptag, GstTagList * taglist,
 
   /* clean up entry */
   g_free (ptag->str);
-  g_slice_free (PendingXmpTag, ptag);
+  g_free (ptag);
   *pending_tags = g_slist_delete_link (*pending_tags, entry);
 }
 
@@ -931,7 +932,7 @@ deserialize_tiff_orientation (XmpTag * xmptag, GstTagList * taglist,
 }
 
 
-/* look at this page for addtional schemas
+/* look at this page for additional schemas
  * http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/XMP.html
  */
 static gpointer
@@ -940,6 +941,8 @@ _init_xmp_tag_map (gpointer user_data)
   XmpTag *xmpinfo;
   GstXmpSchema *schema;
 
+  gst_tag_register_musicbrainz_tags ();
+
   __xmp_schemas = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   /* add the maps */
@@ -1077,7 +1080,7 @@ struct _GstXmpNamespaceMatch
 
   /*
    * Stores extra namespaces for array tags
-   * The namespaces should be writen in the form:
+   * The namespaces should be written in the form:
    *
    * xmlns:XpTo="http://some.org/your/ns/name/ (next ones)"
    */
@@ -1172,7 +1175,7 @@ read_one_tag (GstTagList * list, XmpTag * xmptag,
  *
  * Parse a xmp packet into a taglist.
  *
- * Returns: new taglist or %NULL, free the list when done
+ * Returns: (transfer full) (nullable): new taglist or %NULL, free the list when done
  */
 GstTagList *
 gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
@@ -1353,7 +1356,7 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
                   /* we shouldn't find a xmp structure here */
                   g_assert (xmp_tag->gst_tag != NULL);
 
-                  ptag = g_slice_new (PendingXmpTag);
+                  ptag = g_new (PendingXmpTag, 1);
                   ptag->xmp_tag = xmp_tag;
                   ptag->str = g_strdup (v);
 
@@ -1446,7 +1449,7 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
 
             context_tag = last_xmp_tag;
           } else {
-            ptag = g_slice_new (PendingXmpTag);
+            ptag = g_new (PendingXmpTag, 1);
             ptag->xmp_tag = last_xmp_tag;
             ptag->str = g_strdup (part);
 
@@ -1473,7 +1476,7 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
     read_one_tag (list, ptag->xmp_tag, ptag->str, &pending_tags);
 
     g_free (ptag->str);
-    g_slice_free (PendingXmpTag, ptag);
+    g_free (ptag);
   }
 
   GST_INFO ("xmp packet parsed, %d entries", gst_tag_list_n_tags (list));
@@ -1697,13 +1700,14 @@ write_one_tag (const GstTagList * list, XmpTag * xmp_tag, gpointer user_data)
  * gst_tag_list_to_xmp_buffer:
  * @list: tags
  * @read_only: does the container forbid inplace editing
- * @schemas: %NULL terminated array of schemas to be used on serialization
+ * @schemas: (array zero-terminated=1):
+ *     %NULL terminated array of schemas to be used on serialization
  *
  * Formats a taglist as a xmp packet using only the selected
  * schemas. An empty list (%NULL) means that all schemas should
  * be used
  *
- * Returns: new buffer or %NULL, unref the buffer when done
+ * Returns: (transfer full) (nullable): new buffer or %NULL, unref the buffer when done
  */
 GstBuffer *
 gst_tag_list_to_xmp_buffer (const GstTagList * list, gboolean read_only,
