@@ -23,6 +23,7 @@
 #include "gstvideometa.h"
 
 #include <string.h>
+#include <gst/base/base.h>
 
 /**
  * SECTION:gstvideometa
@@ -31,6 +32,12 @@
  *
  */
 
+static gboolean
+default_map (GstVideoMeta * meta, guint plane, GstMapInfo * info,
+    gpointer * data, gint * stride, GstMapFlags flags);
+static gboolean
+default_unmap (GstVideoMeta * meta, guint plane, GstMapInfo * info);
+
 #ifndef GST_DISABLE_GST_DEBUG
 #define GST_CAT_DEFAULT ensure_debug_category()
 static GstDebugCategory *
@@ -120,7 +127,7 @@ gst_video_meta_transform (GstBuffer * dest, GstMeta * meta,
 GType
 gst_video_meta_api_get_type (void)
 {
-  static volatile GType type = 0;
+  static GType type = 0;
   static const gchar *tags[] =
       { GST_META_TAG_VIDEO_STR, GST_META_TAG_MEMORY_STR,
     GST_META_TAG_VIDEO_COLORSPACE_STR,
@@ -134,6 +141,110 @@ gst_video_meta_api_get_type (void)
   return type;
 }
 
+static gboolean
+video_meta_serialize (const GstMeta * meta, GstByteArrayInterface * data,
+    guint8 * version)
+{
+  GstVideoMeta *vmeta = (GstVideoMeta *) meta;
+
+  if (vmeta->map != default_map || vmeta->unmap != default_unmap) {
+    GST_WARNING ("Cannot serialize video meta with custom map/unmap functions");
+    return FALSE;
+  }
+
+  gsize size = 36 + vmeta->n_planes * 16;
+  guint8 *ptr = gst_byte_array_interface_append (data, size);
+  if (ptr == NULL)
+    return FALSE;
+
+  GstByteWriter bw;
+  gboolean success = TRUE;
+  gst_byte_writer_init_with_data (&bw, ptr, size, FALSE);
+  success &= gst_byte_writer_put_int32_le (&bw, vmeta->flags);
+  success &= gst_byte_writer_put_int32_le (&bw, vmeta->format);
+  success &= gst_byte_writer_put_uint32_le (&bw, vmeta->width);
+  success &= gst_byte_writer_put_uint32_le (&bw, vmeta->height);
+  success &= gst_byte_writer_put_uint32_le (&bw, vmeta->n_planes);
+  for (int n = 0; n < vmeta->n_planes; n++)
+    success &= gst_byte_writer_put_uint64_le (&bw, vmeta->offset[n]);
+  for (int n = 0; n < vmeta->n_planes; n++)
+    success &= gst_byte_writer_put_int32_le (&bw, vmeta->stride[n]);
+  success &= gst_byte_writer_put_uint32_le (&bw, vmeta->alignment.padding_top);
+  success &=
+      gst_byte_writer_put_uint32_le (&bw, vmeta->alignment.padding_bottom);
+  success &= gst_byte_writer_put_uint32_le (&bw, vmeta->alignment.padding_left);
+  success &=
+      gst_byte_writer_put_uint32_le (&bw, vmeta->alignment.padding_right);
+  for (int n = 0; n < vmeta->n_planes; n++)
+    success &=
+        gst_byte_writer_put_uint32_le (&bw, vmeta->alignment.stride_align[n]);
+  g_assert (success);
+
+  return TRUE;
+}
+
+static GstMeta *
+video_meta_deserialize (const GstMetaInfo * info, GstBuffer * buffer,
+    const guint8 * data, gsize size, guint8 version)
+{
+  GstVideoMeta *vmeta = NULL;
+  gint32 flags;
+  gint32 format;
+  guint width;
+  guint height;
+  guint n_planes;
+  GstVideoAlignment align;
+  guint64 offset64[GST_VIDEO_MAX_PLANES];
+  gint32 stride[GST_VIDEO_MAX_PLANES];
+
+  if (version != 0)
+    return NULL;
+
+  GstByteReader br;
+  gboolean success = TRUE;
+  gst_byte_reader_init (&br, data, size);
+  success &= gst_byte_reader_get_int32_le (&br, &flags);
+  success &= gst_byte_reader_get_int32_le (&br, &format);
+  success &= gst_byte_reader_get_uint32_le (&br, &width);
+  success &= gst_byte_reader_get_uint32_le (&br, &height);
+  success &= gst_byte_reader_get_uint32_le (&br, &n_planes);
+
+  if (!success || n_planes > GST_VIDEO_MAX_PLANES)
+    return NULL;
+
+  for (int n = 0; n < n_planes; n++)
+    success &= gst_byte_reader_get_uint64_le (&br, &offset64[n]);
+  for (int n = 0; n < n_planes; n++)
+    success &= gst_byte_reader_get_int32_le (&br, &stride[n]);
+  success &= gst_byte_reader_get_uint32_le (&br, &align.padding_top);
+  success &= gst_byte_reader_get_uint32_le (&br, &align.padding_bottom);
+  success &= gst_byte_reader_get_uint32_le (&br, &align.padding_left);
+  success &= gst_byte_reader_get_uint32_le (&br, &align.padding_right);
+  for (int n = 0; n < n_planes; n++)
+    success &= gst_byte_reader_get_uint32_le (&br, &align.stride_align[n]);
+
+  if (!success)
+    return NULL;
+
+#if GLIB_SIZEOF_SIZE_T != 8
+  gsize offset[GST_VIDEO_MAX_PLANES];
+  for (int i = 0; i < n_planes; i++) {
+    if (offset64[i] > G_MAXSIZE)
+      return NULL;
+    offset[i] = offset64[i];
+  }
+#else
+  gsize *offset = (gsize *) offset64;
+#endif
+
+  vmeta =
+      gst_buffer_add_video_meta_full (buffer, flags, format, width, height,
+      n_planes, offset, stride);
+  gst_video_meta_set_alignment (vmeta, align);
+
+  return (GstMeta *) vmeta;
+}
+
 /* video metadata */
 const GstMetaInfo *
 gst_video_meta_get_info (void)
@@ -141,10 +252,14 @@ gst_video_meta_get_info (void)
   static const GstMetaInfo *video_meta_info = NULL;
 
   if (g_once_init_enter ((GstMetaInfo **) & video_meta_info)) {
-    const GstMetaInfo *meta =
-        gst_meta_register (GST_VIDEO_META_API_TYPE, "GstVideoMeta",
-        sizeof (GstVideoMeta), (GstMetaInitFunction) gst_video_meta_init,
-        (GstMetaFreeFunction) NULL, gst_video_meta_transform);
+    GstMetaInfo *info = gst_meta_info_new (GST_VIDEO_META_API_TYPE,
+        "GstVideoMeta",
+        sizeof (GstVideoMeta));
+    info->init_func = gst_video_meta_init;
+    info->transform_func = gst_video_meta_transform;
+    info->serialize_func = video_meta_serialize;
+    info->deserialize_func = video_meta_deserialize;
+    const GstMetaInfo *meta = gst_meta_info_register (info);
     g_once_init_leave ((GstMetaInfo **) & video_meta_info,
         (GstMetaInfo *) meta);
   }
@@ -160,7 +275,7 @@ gst_video_meta_get_info (void)
  * Buffers can contain multiple #GstVideoMeta metadata items when dealing with
  * multiview buffers.
  *
- * Returns: (transfer none): the #GstVideoMeta with lowest id (usually 0) or %NULL when there
+ * Returns: (transfer none) (nullable): the #GstVideoMeta with lowest id (usually 0) or %NULL when there
  * is no such metadata on @buffer.
  */
 GstVideoMeta *
@@ -193,7 +308,7 @@ gst_buffer_get_video_meta (GstBuffer * buffer)
  * Buffers can contain multiple #GstVideoMeta metadata items when dealing with
  * multiview buffers.
  *
- * Returns: (transfer none): the #GstVideoMeta with @id or %NULL when there is no such metadata
+ * Returns: (transfer none) (nullable): the #GstVideoMeta with @id or %NULL when there is no such metadata
  * on @buffer.
  */
 GstVideoMeta *
@@ -310,8 +425,8 @@ gst_buffer_add_video_meta (GstBuffer * buffer,
 GstVideoMeta *
 gst_buffer_add_video_meta_full (GstBuffer * buffer,
     GstVideoFrameFlags flags, GstVideoFormat format, guint width,
-    guint height, guint n_planes, gsize offset[GST_VIDEO_MAX_PLANES],
-    gint stride[GST_VIDEO_MAX_PLANES])
+    guint height, guint n_planes, const gsize offset[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES])
 {
   GstVideoMeta *meta;
   guint i;
@@ -394,6 +509,25 @@ gst_video_meta_unmap (GstVideoMeta * meta, guint plane, GstMapInfo * info)
   return meta->unmap (meta, plane, info);
 }
 
+static gboolean
+gst_video_meta_is_alignment_valid (GstVideoAlignment * align)
+{
+  gint i;
+
+  g_return_val_if_fail (align != NULL, FALSE);
+
+  if (align->padding_top != 0 || align->padding_bottom != 0 ||
+      align->padding_left != 0 || align->padding_right != 0)
+    return TRUE;
+
+  for (i = 0; i < GST_VIDEO_MAX_PLANES; i++) {
+    if (align->stride_align[i] != 0)
+      return TRUE;
+  }
+
+  return FALSE;
+}
+
 static gboolean
 gst_video_meta_validate_alignment (GstVideoMeta * meta,
     gsize plane_size[GST_VIDEO_MAX_PLANES])
@@ -401,6 +535,15 @@ gst_video_meta_validate_alignment (GstVideoMeta * meta,
   GstVideoInfo info;
   guint i;
 
+  if (!gst_video_meta_is_alignment_valid (&meta->alignment)) {
+    GST_LOG ("Set alignment on meta to all zero");
+
+    /* When alignment is invalid, no further check is needed,
+       unless user wants to calculate the pitch for each plane. */
+    if (!plane_size)
+      return TRUE;
+  }
+
   gst_video_info_init (&info);
   gst_video_info_set_format (&info, meta->format, meta->width, meta->height);
 
@@ -460,7 +603,7 @@ gst_video_meta_set_alignment (GstVideoMeta * meta, GstVideoAlignment alignment)
 /**
  * gst_video_meta_get_plane_size:
  * @meta: a #GstVideoMeta
- * @plane_size: (out): array used to store the plane sizes
+ * @plane_size: (out caller-allocates) (array fixed-size=4): array used to store the plane sizes
  *
  * Compute the size, in bytes, of each video plane described in @meta including
  * any padding and alignment constraint defined in @meta->alignment.
@@ -483,7 +626,7 @@ gst_video_meta_get_plane_size (GstVideoMeta * meta,
 /**
  * gst_video_meta_get_plane_height:
  * @meta: a #GstVideoMeta
- * @plane_height: (out): array used to store the plane height
+ * @plane_height: (out caller-allocates) (array fixed-size=4): array used to store the plane height
  *
  * Compute the padded height of each plane from @meta (padded size
  * divided by stride).
@@ -577,7 +720,7 @@ gst_video_crop_meta_transform (GstBuffer * dest, GstMeta * meta,
 GType
 gst_video_crop_meta_api_get_type (void)
 {
-  static volatile GType type = 0;
+  static GType type = 0;
   static const gchar *tags[] =
       { GST_META_TAG_VIDEO_STR, GST_META_TAG_VIDEO_SIZE_STR,
     GST_META_TAG_VIDEO_ORIENTATION_STR, NULL
@@ -638,7 +781,7 @@ gst_video_meta_transform_scale_get_quark (void)
 GType
 gst_video_gl_texture_upload_meta_api_get_type (void)
 {
-  static volatile GType type = 0;
+  static GType type = 0;
   static const gchar *tags[] =
       { GST_META_TAG_VIDEO_STR, GST_META_TAG_MEMORY_STR, NULL };
 
@@ -806,7 +949,7 @@ gst_video_gl_texture_upload_meta_upload (GstVideoGLTextureUploadMeta * meta,
 GType
 gst_video_region_of_interest_meta_api_get_type (void)
 {
-  static volatile GType type;
+  static GType type;
   static const gchar *tags[] =
       { GST_META_TAG_VIDEO_STR, GST_META_TAG_VIDEO_ORIENTATION_STR,
     GST_META_TAG_VIDEO_SIZE_STR, NULL
@@ -924,7 +1067,7 @@ gst_video_region_of_interest_meta_get_info (void)
  * Buffers can contain multiple #GstVideoRegionOfInterestMeta metadata items if
  * multiple regions of interests are marked on a frame.
  *
- * Returns: (transfer none): the #GstVideoRegionOfInterestMeta with @id or %NULL when there is
+ * Returns: (transfer none) (nullable): the #GstVideoRegionOfInterestMeta with @id or %NULL when there is
  * no such metadata on @buffer.
  */
 GstVideoRegionOfInterestMeta *
@@ -1064,7 +1207,7 @@ gst_video_region_of_interest_meta_get_param (GstVideoRegionOfInterestMeta *
 GType
 gst_video_time_code_meta_api_get_type (void)
 {
-  static volatile GType type;
+  static GType type;
 
   if (g_once_init_enter (&type)) {
     static const gchar *tags[] = { NULL };
@@ -1178,7 +1321,7 @@ gst_buffer_add_video_time_code_meta (GstBuffer * buffer,
  * Attaches #GstVideoTimeCodeMeta metadata to @buffer with the given
  * parameters.
  *
- * Returns: (transfer none): the #GstVideoTimeCodeMeta on @buffer, or
+ * Returns: (transfer none) (nullable): the #GstVideoTimeCodeMeta on @buffer, or
  * (since 1.16) %NULL if the timecode was invalid.
  *
  * Since: 1.10
