@@ -831,18 +831,8 @@ _create_compatible_processor (GList * all_processors,
 {
   GList *processors1, *processors, *tmp;
   GstElement *processor = NULL;
+  GstElementFactory *factory = NULL;
   GstCaps *format = NULL;
-  GstCaps *encoding_format = NULL;
-  GstStructure *s;
-  const gchar *encoding_media_type;
-
-  encoding_format = gst_encoding_profile_get_format (sprof);
-  if (G_UNLIKELY (gst_caps_is_empty (encoding_format))) {
-    return NULL;
-  }
-
-  s = gst_caps_get_structure (encoding_format, 0);
-  encoding_media_type = gst_structure_get_name (s);
 
   if (encoder) {
     GstPadTemplate *template = gst_element_get_pad_template (encoder, "src");
@@ -853,7 +843,7 @@ _create_compatible_processor (GList * all_processors,
 
   if (!format || gst_caps_is_any (format)) {
     gst_clear_caps (&format);
-    format = gst_caps_ref (encoding_format);
+    format = gst_encoding_profile_get_format (sprof);
   }
 
   GST_DEBUG ("Getting list of processors for format %" GST_PTR_FORMAT, format);
@@ -874,52 +864,29 @@ _create_compatible_processor (GList * all_processors,
   }
 
   for (tmp = processors; tmp; tmp = tmp->next) {
-    GstElementFactory *candidate_factory = GST_ELEMENT_FACTORY_CAST (tmp->data);
-    GstPadTemplate *tmpl;
-    GstCaps *processor_caps;
-    gboolean is_compatible = FALSE;
-
-    processor = gst_element_factory_create (candidate_factory, NULL);
-    tmpl = gst_element_get_pad_template (processor, "sink");
-    processor_caps = gst_pad_template_get_caps (tmpl);
-
-    if (gst_caps_is_any (processor_caps)) {
-      is_compatible = TRUE;
-    } else if (!gst_caps_is_empty (processor_caps)) {
-      GstStructure *structure = gst_caps_get_structure (processor_caps, 0);
-      if (!strcmp (encoding_media_type, gst_structure_get_name (structure))) {
-        is_compatible = TRUE;
-      }
-    }
-
-    gst_clear_caps (&processor_caps);
-
-    if (!is_compatible) {
-      GST_DEBUG ("Processor %" GST_PTR_FORMAT " is not compatible with format %"
-          GST_PTR_FORMAT, processor, encoding_format);
-      gst_clear_object (&processor);
-      continue;
-    }
-
-    /* FIXME : signal the user if he wants this parser */
+    /* FIXME : We're only picking the first one so far */
+    /* FIXME : signal the user if he wants this */
+    factory = (GstElementFactory *) tmp->data;
     break;
   }
 
+  if (factory)
+    processor = gst_element_factory_create (factory, NULL);
+
   gst_plugin_feature_list_free (processors);
 
 beach:
-  gst_clear_caps (&format);
-  gst_clear_caps (&encoding_format);
+  if (format)
+    gst_caps_unref (format);
 
   return processor;
 }
 
 static gboolean
-_set_properties (const GstIdStr * property, const GValue * value,
-    GObject * element)
+_set_properties (GQuark property_id, const GValue * value, GObject * element)
 {
-  GST_DEBUG_OBJECT (element, "Setting %s", gst_id_str_as_str (property));
-  g_object_set_property (element, gst_id_str_as_str (property), value);
+  GST_DEBUG_OBJECT (element, "Setting %s", g_quark_to_string (property_id));
+  g_object_set_property (element, g_quark_to_string (property_id), value);
 
   return TRUE;
 }
@@ -938,8 +905,8 @@ set_element_properties_from_encoding_profile (GstEncodingProfile * profile,
     return;
 
   if (!gst_structure_has_name (properties, "element-properties-map")) {
-    gst_structure_foreach_id_str (properties,
-        (GstStructureForeachIdStrFunc) _set_properties, element);
+    gst_structure_foreach (properties,
+        (GstStructureForeachFunc) _set_properties, element);
     goto done;
   }
 
@@ -971,8 +938,8 @@ set_element_properties_from_encoding_profile (GstEncodingProfile * profile,
     GST_DEBUG_OBJECT (GST_OBJECT_PARENT (element),
         "Setting %" GST_PTR_FORMAT " on %" GST_PTR_FORMAT, tmp_properties,
         element);
-    gst_structure_foreach_id_str (tmp_properties,
-        (GstStructureForeachIdStrFunc) _set_properties, element);
+    gst_structure_foreach (tmp_properties,
+        (GstStructureForeachFunc) _set_properties, element);
     goto done;
   }
 
@@ -1133,7 +1100,7 @@ static inline GstPad *
 get_compatible_muxer_sink_pad (GstEncodeBaseBin * ebin,
     GstEncodingProfile * sprof, GstCaps * sinkcaps)
 {
-  GstPad *sinkpad = NULL;
+  GstPad *sinkpad;
   GList *padl, *compatible_templates = NULL;
 
   GST_DEBUG_OBJECT (ebin, "Finding muxer pad for caps: %" GST_PTR_FORMAT,
@@ -1595,7 +1562,6 @@ _create_stream_group (GstEncodeBaseBin * ebin, GstEncodingProfile * sprof,
     gst_object_unref (muxerpad);
   } else {
     if (ebin->srcpad) {
-      /* encodebin static source pad */
       gst_ghost_pad_set_target (GST_GHOST_PAD (ebin->srcpad), srcpad);
     } else {
       if (!gst_encode_base_bin_create_src_pad (ebin, srcpad)) {
@@ -1667,7 +1633,6 @@ _create_stream_group (GstEncodeBaseBin * ebin, GstEncodingProfile * sprof,
               (sgroup->parser)), NULL);
 
       gst_bin_add (GST_BIN (ebin), p1);
-      tosync = g_list_append (tosync, p1);
       if (G_UNLIKELY (!gst_element_link (p1, last)))
         goto parser_link_failure;
 
@@ -2062,12 +2027,10 @@ cleanup:
 }
 
 static gboolean
-_gst_caps_match_foreach (const GstIdStr * fieldname, const GValue * value,
-    gpointer data)
+_gst_caps_match_foreach (GQuark field_id, const GValue * value, gpointer data)
 {
   GstStructure *structure = data;
-  const GValue *other_value =
-      gst_structure_id_str_get_value (structure, fieldname);
+  const GValue *other_value = gst_structure_id_get_value (structure, field_id);
 
   if (G_UNLIKELY (other_value == NULL))
     return FALSE;
@@ -2093,7 +2056,7 @@ _gst_caps_match (const GstCaps * caps_a, const GstCaps * caps_b)
     for (j = 0; j < gst_caps_get_size (caps_b); j++) {
       GstStructure *structure_b = gst_caps_get_structure (caps_b, j);
 
-      res = gst_structure_foreach_id_str (structure_a, _gst_caps_match_foreach,
+      res = gst_structure_foreach (structure_a, _gst_caps_match_foreach,
           structure_b);
       if (res)
         goto end;
@@ -2308,7 +2271,6 @@ create_elements_and_pads (GstEncodeBaseBin * ebin)
      * but for the time being let's assume it's a static pad :) */
     muxerpad = gst_element_get_static_pad (muxer, "src");
     if (ebin->srcpad) {
-      /* encodebin static source pad */
       if (G_UNLIKELY (muxerpad == NULL))
         goto no_muxer_pad;
       if (!gst_ghost_pad_set_target (GST_GHOST_PAD (ebin->srcpad), muxerpad))
@@ -2463,9 +2425,7 @@ stream_group_free (GstEncodeBaseBin * ebin, StreamGroup * sgroup)
   if (sgroup->parser) {
     gst_element_set_state (sgroup->parser, GST_STATE_NULL);
     gst_element_unlink (sgroup->parser, sgroup->outfilter);
-    if (sgroup->combiner) {
-      gst_element_unlink (sgroup->combiner, sgroup->parser);
-    }
+    gst_element_unlink (sgroup->combiner, sgroup->parser);
     gst_bin_remove ((GstBin *) ebin, sgroup->parser);
   }
 
@@ -2590,7 +2550,7 @@ gst_encode_base_bin_tear_down_profile (GstEncodeBaseBin * ebin)
     stream_group_remove (ebin, (StreamGroup *) ebin->streams->data);
 
   if (ebin->srcpad) {
-    /* encodebin static source pad, set ghostpad target to NULL */
+    /* Set ghostpad target to NULL */
     gst_ghost_pad_set_target (GST_GHOST_PAD (ebin->srcpad), NULL);
   }
 
@@ -2603,8 +2563,7 @@ gst_encode_base_bin_tear_down_profile (GstEncodeBaseBin * ebin)
     ebin->muxer = NULL;
   }
 
-  if (!ebin->srcpad) {
-    /* encodebin2 dynamic source pads */
+  if (!element->srcpads) {
     while (element->srcpads)
       gst_element_remove_pad (element, element->srcpads->data);
   }
