@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2007 Rene Stadler <mail@renestadler.de>
  * Copyright (C) 2007-2009 Sebastian Dröge <sebastian.droege@collabora.co.uk>
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
@@ -24,10 +24,19 @@
 #endif
 
 #include "gstgiobasesink.h"
+#include "gstgioelements.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_gio_base_sink_debug);
 #define GST_CAT_DEFAULT gst_gio_base_sink_debug
 
+#define DEFAULT_CLOSE_ON_STOP FALSE
+
+enum
+{
+  PROP_0,
+  PROP_CLOSE_ON_STOP,
+};
+
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -37,6 +46,10 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
 G_DEFINE_TYPE (GstGioBaseSink, gst_gio_base_sink, GST_TYPE_BASE_SINK);
 
 static void gst_gio_base_sink_finalize (GObject * object);
+static void gst_gio_base_sink_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
+static void gst_gio_base_sink_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
 static gboolean gst_gio_base_sink_start (GstBaseSink * base_sink);
 static gboolean gst_gio_base_sink_stop (GstBaseSink * base_sink);
 static gboolean gst_gio_base_sink_unlock (GstBaseSink * base_sink);
@@ -58,9 +71,26 @@ gst_gio_base_sink_class_init (GstGioBaseSinkClass * klass)
       "GIO base sink");
 
   gobject_class->finalize = gst_gio_base_sink_finalize;
+  gobject_class->get_property = gst_gio_base_sink_get_property;
+  gobject_class->set_property = gst_gio_base_sink_set_property;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_factory);
+
+  /**
+   * GstGioBaseSink:close-on-stop:
+   *
+   * Determines whether the stream is closed with the element "stops" (i.e. goes
+   * from the READY to NULL state), or when the element is disposed. Which of
+   * these behaviours is desirable depends on the lifecycle of the underlying
+   * stream that the element works with.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class, PROP_CLOSE_ON_STOP,
+      g_param_spec_boolean ("close-on-stop", "Close stream on stop",
+          "Close the stream when the element stops (i.e. goes from READY to "
+          "NULL) rather than when the element is disposed)",
+          DEFAULT_CLOSE_ON_STOP, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstbasesink_class->start = GST_DEBUG_FUNCPTR (gst_gio_base_sink_start);
   gstbasesink_class->stop = GST_DEBUG_FUNCPTR (gst_gio_base_sink_stop);
@@ -70,6 +100,8 @@ gst_gio_base_sink_class_init (GstGioBaseSinkClass * klass)
   gstbasesink_class->query = GST_DEBUG_FUNCPTR (gst_gio_base_sink_query);
   gstbasesink_class->event = GST_DEBUG_FUNCPTR (gst_gio_base_sink_event);
   gstbasesink_class->render = GST_DEBUG_FUNCPTR (gst_gio_base_sink_render);
+
+  gst_type_mark_as_plugin_api (GST_TYPE_GIO_BASE_SINK, 0);
 }
 
 static void
@@ -78,6 +110,8 @@ gst_gio_base_sink_init (GstGioBaseSink * sink)
   gst_base_sink_set_sync (GST_BASE_SINK (sink), FALSE);
 
   sink->cancel = g_cancellable_new ();
+  // FALSE is the historical default for this class
+  sink->close_on_stop = DEFAULT_CLOSE_ON_STOP;
 }
 
 static void
@@ -98,6 +132,38 @@ gst_gio_base_sink_finalize (GObject * object)
   GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
+static void
+gst_gio_base_sink_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstGioBaseSink *sink = GST_GIO_BASE_SINK (object);
+
+  switch (prop_id) {
+    case PROP_CLOSE_ON_STOP:
+      g_value_set_boolean (value, sink->close_on_stop);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_gio_base_sink_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstGioBaseSink *sink = GST_GIO_BASE_SINK (object);
+
+  switch (prop_id) {
+    case PROP_CLOSE_ON_STOP:
+      sink->close_on_stop = g_value_get_boolean (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 static gboolean
 gst_gio_base_sink_start (GstBaseSink * base_sink)
 {
@@ -127,11 +193,10 @@ static gboolean
 gst_gio_base_sink_stop (GstBaseSink * base_sink)
 {
   GstGioBaseSink *sink = GST_GIO_BASE_SINK (base_sink);
-  GstGioBaseSinkClass *klass = GST_GIO_BASE_SINK_GET_CLASS (sink);
   gboolean success;
   GError *err = NULL;
 
-  if (klass->close_on_stop && G_IS_OUTPUT_STREAM (sink->stream)) {
+  if (sink->close_on_stop && G_IS_OUTPUT_STREAM (sink->stream)) {
     GST_DEBUG_OBJECT (sink, "closing stream");
 
     /* FIXME: can block but unfortunately we can't use async operations
@@ -191,7 +256,8 @@ gst_gio_base_sink_unlock_stop (GstBaseSink * base_sink)
 
   GST_LOG_OBJECT (sink, "resetting cancellable");
 
-  g_cancellable_reset (sink->cancel);
+  g_object_unref (sink->cancel);
+  sink->cancel = g_cancellable_new ();
 
   return TRUE;
 }
@@ -261,7 +327,7 @@ static GstFlowReturn
 gst_gio_base_sink_render (GstBaseSink * base_sink, GstBuffer * buffer)
 {
   GstGioBaseSink *sink = GST_GIO_BASE_SINK (base_sink);
-  gssize written;
+  gsize written;
   GstMapInfo map;
   gboolean success;
   GError *err = NULL;
@@ -274,23 +340,11 @@ gst_gio_base_sink_render (GstBaseSink * base_sink, GstBuffer * buffer)
       "writing %" G_GSIZE_FORMAT " bytes to offset %" G_GUINT64_FORMAT,
       map.size, sink->position);
 
-  written =
-      g_output_stream_write (sink->stream, map.data, map.size, sink->cancel,
-      &err);
+  success =
+      g_output_stream_write_all (sink->stream, map.data, map.size, &written,
+      sink->cancel, &err);
   gst_buffer_unmap (buffer, &map);
 
-  success = (written >= 0);
-
-  if (G_UNLIKELY (success && written < map.size)) {
-    /* FIXME: Can this happen?  Should we handle it gracefully?  gnomevfssink
-     * doesn't... */
-    GST_ELEMENT_ERROR (sink, RESOURCE, WRITE, (NULL),
-        ("Could not write to stream: (short write, only %"
-            G_GSSIZE_FORMAT " bytes of %" G_GSIZE_FORMAT " bytes written)",
-            written, map.size));
-    return GST_FLOW_ERROR;
-  }
-
   if (success) {
     sink->position += written;
     return GST_FLOW_OK;
@@ -298,7 +352,7 @@ gst_gio_base_sink_render (GstBaseSink * base_sink, GstBuffer * buffer)
   } else {
     GstFlowReturn ret;
 
-    if (!gst_gio_error (sink, "g_output_stream_write", &err, &ret)) {
+    if (!gst_gio_error (sink, "g_output_stream_write_all", &err, &ret)) {
       if (GST_GIO_ERROR_MATCHES (err, NO_SPACE)) {
         GST_ELEMENT_ERROR (sink, RESOURCE, NO_SPACE_LEFT, (NULL),
             ("Could not write to stream: %s", err->message));
