@@ -223,6 +223,7 @@ struct _GstURIDecodeBin3
   guint64 buffer_duration;      /* When buffering, buffer duration (ns) */
   guint buffer_size;            /* When buffering, buffer size (bytes) */
   gboolean download;
+  gchar *download_dir;
   gboolean use_buffering;
   guint64 ring_buffer_max_size;
   gboolean instant_uri;         /* Whether URI changes should be applied immediately or not */
@@ -313,6 +314,7 @@ enum
   PROP_BUFFER_SIZE,
   PROP_BUFFER_DURATION,
   PROP_DOWNLOAD,
+  PROP_DOWNLOAD_DIR,
   PROP_USE_BUFFERING,
   PROP_RING_BUFFER_MAX_SIZE,
   PROP_CAPS,
@@ -470,6 +472,23 @@ gst_uri_decode_bin3_class_init (GstURIDecodeBin3Class * klass)
       g_param_spec_boolean ("download", "Download",
           "Attempt download buffering when buffering network streams",
           DEFAULT_DOWNLOAD, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstURIDecodeBin3:download-dir:
+   *
+   * The directory where buffers are downloaded to, if 'download' is enabled.
+   * If not set (default), the XDG cache directory is used.
+   * Will be applied to the next 'uri' played or until the element go back to
+   * the PAUSED state.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class, PROP_DOWNLOAD_DIR,
+      g_param_spec_string ("download-dir", "Download Directory",
+          "The directory where buffers are downloaded to, if 'download' is enabled. "
+          "If not set (default), the XDG cache directory is used.",
+          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GstURIDecodeBin3::use-buffering:
    *
@@ -582,11 +601,21 @@ gst_uri_decode_bin3_class_init (GstURIDecodeBin3Class * klass)
   klass->select_stream = gst_uridecodebin3_select_stream;
 }
 
+static void
+current_updated (GstURIDecodeBin3 * dec)
+{
+  GObject *object = G_OBJECT (dec);
+
+  g_object_notify (object, "current-uri");
+  g_object_notify (object, "current-suburi");
+}
+
 static void
 check_output_group_id (GstURIDecodeBin3 * dec)
 {
   GList *iter;
   guint common_group_id = GST_GROUP_ID_INVALID;
+  gboolean notify_current = FALSE;
 
   PLAY_ITEMS_LOCK (dec);
 
@@ -619,9 +648,15 @@ check_output_group_id (GstURIDecodeBin3 * dec)
       dec->output_item->group_id = common_group_id;
       free_play_item (dec, previous_item);
     }
+    notify_current = TRUE;
   }
 
   PLAY_ITEMS_UNLOCK (dec);
+
+  if (notify_current) {
+    /* don't hold the object lock as application could fetch some properties whose getters require this lock as well */
+    current_updated (dec);
+  }
 }
 
 static GstPadProbeReturn
@@ -675,7 +710,7 @@ add_output_pad (GstURIDecodeBin3 * dec, GstPad * target_pad)
   gchar *pad_name;
   GstEvent *stream_start;
 
-  output = g_slice_new0 (OutputPad);
+  output = g_new0 (OutputPad, 1);
 
   GST_LOG_OBJECT (dec, "Created output %p", output);
 
@@ -751,7 +786,7 @@ db_pad_removed_cb (GstElement * element, GstPad * pad, GstURIDecodeBin3 * dec)
     /* Remove event probe */
     gst_pad_remove_probe (output->target_pad, output->probe_id);
 
-    g_slice_free (OutputPad, output);
+    g_free (output);
 
     check_output_group_id (dec);
   }
@@ -903,6 +938,7 @@ gst_uri_decode_bin3_dispose (GObject * obj)
   }
   g_list_free (dec->play_items);
   dec->play_items = NULL;
+  g_clear_pointer (&dec->download_dir, g_free);
 
   g_mutex_clear (&dec->play_items_lock);
 
@@ -1059,6 +1095,7 @@ switch_and_activate_input_locked (GstURIDecodeBin3 * uridecodebin,
   GList *old_pads = get_all_play_item_source_pads (uridecodebin->input_item);
   GList *to_activate = NULL;
   GList *iternew, *iterold;
+  gboolean inactive_previous_item = old_pads == NULL;
 
   /* Deactivate old urisourcebins first ? Problem is they might remove the pads */
 
@@ -1078,7 +1115,7 @@ switch_and_activate_input_locked (GstURIDecodeBin3 * uridecodebin,
       new_spad->db3_pad_is_request = old_spad->db3_pad_is_request;
       old_spad->db3_sink_pad = NULL;
 
-      gst_pad_link (new_spad->src_pad, new_spad->db3_sink_pad);
+      /* NOTE : Pad will be linked further down */
       old_pads = g_list_remove (old_pads, old_spad);
     } else {
       GST_DEBUG_OBJECT (new_spad->src_pad, "Needs a new pad");
@@ -1086,6 +1123,33 @@ switch_and_activate_input_locked (GstURIDecodeBin3 * uridecodebin,
     }
   }
 
+  /* If the old pads contains the static decodebin3 sinkpad *and* we have a new
+   *  pad to activate, we re-use it */
+  if (to_activate) {
+    /* Remove unmatched old source pads */
+    for (iterold = old_pads; iterold; iterold = iterold->next) {
+      GstSourcePad *old_spad = iterold->data;
+      if (old_spad->db3_sink_pad && !old_spad->db3_pad_is_request) {
+        GstSourcePad *new_spad = to_activate->data;
+
+        GST_DEBUG_OBJECT (uridecodebin, "Static sinkpad can be re-used");
+        GST_DEBUG_OBJECT (uridecodebin, "Relinking %s:%s from %s:%s to %s:%s",
+            GST_DEBUG_PAD_NAME (old_spad->db3_sink_pad),
+            GST_DEBUG_PAD_NAME (old_spad->src_pad),
+            GST_DEBUG_PAD_NAME (new_spad->src_pad));
+        gst_pad_unlink (old_spad->src_pad, old_spad->db3_sink_pad);
+        new_spad->db3_sink_pad = old_spad->db3_sink_pad;
+        new_spad->db3_pad_is_request = old_spad->db3_pad_is_request;
+        old_spad->db3_sink_pad = NULL;
+
+        /* NOTE : Pad will be linked further down */
+        old_pads = g_list_remove (old_pads, old_spad);
+        to_activate = g_list_remove (to_activate, new_spad);
+        break;
+      }
+    }
+  }
+
   /* Remove unmatched old source pads */
   for (iterold = old_pads; iterold; iterold = iterold->next) {
     GstSourcePad *old_spad = iterold->data;
@@ -1097,7 +1161,14 @@ switch_and_activate_input_locked (GstURIDecodeBin3 * uridecodebin,
     }
   }
 
-  /* Link new source pads */
+  /* Link existing pads */
+  for (iternew = new_pads; iternew; iternew = iternew->next) {
+    GstSourcePad *new_spad = iternew->data;
+    if (new_spad->db3_sink_pad)
+      gst_pad_link (new_spad->src_pad, new_spad->db3_sink_pad);
+  }
+
+  /* Request (and link) new pads */
   for (iternew = to_activate; iternew; iternew = iternew->next) {
     GstSourcePad *new_spad = iternew->data;
     link_src_pad_to_db3 (uridecodebin, new_spad);
@@ -1126,6 +1197,17 @@ switch_and_activate_input_locked (GstURIDecodeBin3 * uridecodebin,
     uridecodebin->input_item->sub_item = NULL;
   }
 
+  /* If the previous play item was not active at all (i.e. was never linked to
+   * decodebin3), this one *also* becomes the output one */
+  if (inactive_previous_item) {
+    GST_DEBUG_OBJECT (uridecodebin,
+        "Previous play item was never activated, discarding");
+    uridecodebin->play_items =
+        g_list_remove (uridecodebin->play_items, uridecodebin->input_item);
+    free_play_item (uridecodebin, uridecodebin->input_item);
+    uridecodebin->output_item = new_item;
+  }
+
   /* and set new one as input item */
   uridecodebin->input_item = new_item;
 
@@ -1185,10 +1267,12 @@ uri_src_probe (GstPad * pad, GstPadProbeInfo * info, GstSourcePad * srcpad)
     case GST_EVENT_STREAM_START:
     {
       GstStream *stream = NULL;
+      GstQuery *q = gst_query_new_selectable ();
       guint group_id = GST_GROUP_ID_INVALID;
 
       srcpad->saw_eos = FALSE;
       gst_event_parse_group_id (event, &group_id);
+
       /* Unify group id */
       if (handler->play_item->group_id == GST_GROUP_ID_INVALID) {
         GST_DEBUG_OBJECT (pad,
@@ -1201,14 +1285,45 @@ uri_src_probe (GstPad * pad, GstPadProbeInfo * info, GstSourcePad * srcpad)
         GST_PAD_PROBE_INFO_DATA (info) = event;
         gst_event_set_group_id (event, handler->play_item->group_id);
       }
+
       gst_event_parse_stream (event, &stream);
       if (stream) {
         GST_DEBUG_OBJECT (srcpad->src_pad, "Got GstStream %" GST_PTR_FORMAT,
             stream);
-        if (srcpad->stream)
-          gst_object_unref (srcpad->stream);
-        srcpad->stream = stream;
+        gst_object_replace ((GstObject **) & srcpad->stream,
+            (GstObject *) stream);
       }
+
+      /* Remember whether upstream is selectable or not */
+      if (gst_pad_query (pad, q)) {
+        PLAY_ITEMS_LOCK (handler->uridecodebin);
+        gst_query_parse_selectable (q, &handler->upstream_selected);
+        GST_DEBUG_OBJECT (srcpad->src_pad, "Upstream is selectable : %d",
+            handler->upstream_selected);
+        PLAY_ITEMS_UNLOCK (handler->uridecodebin);
+      }
+      gst_query_unref (q);
+
+      break;
+    }
+    case GST_EVENT_STREAM_COLLECTION:
+    {
+      GstStreamCollection *collection = NULL;
+      PLAY_ITEMS_LOCK (handler->uridecodebin);
+      if (!handler->upstream_selected) {
+        gst_event_parse_stream_collection (event, &collection);
+        if (collection) {
+          GST_DEBUG_OBJECT (srcpad->src_pad, "Seen collection with %d streams",
+              gst_stream_collection_get_size (collection));
+          if (handler->expected_pads == 1) {
+            handler->expected_pads =
+                gst_stream_collection_get_size (collection);
+          }
+          gst_object_unref (collection);
+        }
+      }
+      PLAY_ITEMS_UNLOCK (handler->uridecodebin);
+
       break;
     }
     case GST_EVENT_SEGMENT:
@@ -1229,57 +1344,16 @@ uri_src_block_probe (GstPad * pad, GstPadProbeInfo * info,
 {
   GstPadProbeReturn ret = GST_PAD_PROBE_OK;
   GstSourceHandler *handler = srcpad->handler;
-  GST_DEBUG_OBJECT (pad, "blocked");
 
   /* We only block on buffers, buffer list and gap events. Everything else is
-   * dropped (sticky events will be propagated later) */
+   * passed to the regular (non-blocking) probe */
   if (GST_IS_EVENT (GST_PAD_PROBE_INFO_DATA (info)) &&
       GST_EVENT_TYPE (GST_PAD_PROBE_INFO_EVENT (info)) != GST_EVENT_GAP) {
-    GstEvent *event = GST_PAD_PROBE_INFO_EVENT (info);
-    if (GST_EVENT_TYPE (event) == GST_EVENT_STREAM_START) {
-      GstStream *stream = NULL;
-      GstQuery *q = gst_query_new_selectable ();
-      gst_event_parse_stream (event, &stream);
-      if (stream) {
-        GST_DEBUG_OBJECT (srcpad->src_pad, "Got GstStream %" GST_PTR_FORMAT,
-            stream);
-        if (srcpad->stream)
-          gst_object_unref (srcpad->stream);
-        srcpad->stream = stream;
-      }
-      if (gst_pad_query (pad, q)) {
-        PLAY_ITEMS_LOCK (handler->uridecodebin);
-        gst_query_parse_selectable (q, &handler->upstream_selected);
-        GST_DEBUG_OBJECT (srcpad->src_pad, "Upstream is selectable : %d",
-            handler->upstream_selected);
-        PLAY_ITEMS_UNLOCK (handler->uridecodebin);
-      }
-      gst_query_unref (q);
-    } else if (GST_EVENT_TYPE (event) == GST_EVENT_STREAM_COLLECTION) {
-      GstStreamCollection *collection = NULL;
-      PLAY_ITEMS_LOCK (handler->uridecodebin);
-      if (!handler->upstream_selected) {
-        gst_event_parse_stream_collection (event, &collection);
-        if (collection) {
-          GST_DEBUG_OBJECT (srcpad->src_pad, "Seen collection with %d streams",
-              gst_stream_collection_get_size (collection));
-          if (handler->expected_pads == 1) {
-            handler->expected_pads =
-                gst_stream_collection_get_size (collection);
-          }
-          gst_object_unref (collection);
-        }
-      }
-      PLAY_ITEMS_UNLOCK (handler->uridecodebin);
-    }
-
-    GST_LOG_OBJECT (pad, "Skiping %" GST_PTR_FORMAT, event);
-    /* We don't want to be repeatedly called for the same event when unlinked,
-     * so we mark the event as handled */
-    gst_mini_object_unref (GST_PAD_PROBE_INFO_DATA (info));
-    return GST_PAD_PROBE_HANDLED;
+    return GST_PAD_PROBE_PASS;
   }
 
+  GST_DEBUG_OBJECT (pad, "blocking on buffer or gap");
+
   PLAY_ITEMS_LOCK (handler->uridecodebin);
   if (srcpad->block_probe_id == 0) {
     GST_DEBUG_OBJECT (pad, "pad has already been unblocked");
@@ -1331,7 +1405,7 @@ static void
 src_pad_added_cb (GstElement * element, GstPad * pad,
     GstSourceHandler * handler)
 {
-  GstSourcePad *spad = g_slice_new0 (GstSourcePad);
+  GstSourcePad *spad = g_new0 (GstSourcePad, 1);
   GstURIDecodeBin3 *uridecodebin;
 
   uridecodebin = handler->uridecodebin;
@@ -1411,7 +1485,7 @@ src_pad_removed_cb (GstElement * element, GstPad * pad,
     gst_object_unref (spad->stream);
 
   handler->sourcepads = g_list_remove (handler->sourcepads, spad);
-  g_slice_free (GstSourcePad, spad);
+  g_free (spad);
 }
 
 static void
@@ -1437,7 +1511,7 @@ new_source_handler (GstURIDecodeBin3 * uridecodebin, GstPlayItem * item,
 {
   GstSourceHandler *handler;
 
-  handler = g_slice_new0 (GstSourceHandler);
+  handler = g_new0 (GstSourceHandler, 1);
 
   handler->uridecodebin = uridecodebin;
   handler->play_item = item;
@@ -1447,6 +1521,7 @@ new_source_handler (GstURIDecodeBin3 * uridecodebin, GstPlayItem * item,
   g_object_set (handler->urisourcebin,
       "connection-speed", uridecodebin->connection_speed / 1000,
       "download", uridecodebin->download,
+      "download-dir", uridecodebin->download_dir,
       "use-buffering", uridecodebin->use_buffering,
       "buffer-duration", uridecodebin->buffer_duration,
       "buffer-size", uridecodebin->buffer_size,
@@ -1506,7 +1581,9 @@ gst_uri_decode_bin3_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_URI:
+      PLAY_ITEMS_LOCK (dec);
       gst_uri_decode_bin3_set_uri (dec, g_value_get_string (value));
+      PLAY_ITEMS_UNLOCK (dec);
       break;
     case PROP_SUBURI:
       gst_uri_decode_bin3_set_suburi (dec, g_value_get_string (value));
@@ -1525,6 +1602,9 @@ gst_uri_decode_bin3_set_property (GObject * object, guint prop_id,
     case PROP_DOWNLOAD:
       dec->download = g_value_get_boolean (value);
       break;
+    case PROP_DOWNLOAD_DIR:
+      dec->download_dir = g_value_dup_string (value);
+      break;
     case PROP_USE_BUFFERING:
       dec->use_buffering = g_value_get_boolean (value);
       break;
@@ -1619,6 +1699,9 @@ gst_uri_decode_bin3_get_property (GObject * object, guint prop_id,
     case PROP_DOWNLOAD:
       g_value_set_boolean (value, dec->download);
       break;
+    case PROP_DOWNLOAD_DIR:
+      g_value_set_string (value, dec->download_dir);
+      break;
     case PROP_USE_BUFFERING:
       g_value_set_boolean (value, dec->use_buffering);
       break;
@@ -1667,13 +1750,13 @@ free_source_handler (GstURIDecodeBin3 * uridecodebin,
   }
   if (handler->pending_buffering_msg)
     gst_message_unref (handler->pending_buffering_msg);
-  g_slice_free (GstSourceHandler, handler);
+  g_free (handler);
 }
 
 static GstSourceItem *
 new_source_item (GstURIDecodeBin3 * dec, GstPlayItem * item, gchar * uri)
 {
-  GstSourceItem *sourceitem = g_slice_new0 (GstSourceItem);
+  GstSourceItem *sourceitem = g_new0 (GstSourceItem, 1);
 
   sourceitem->play_item = item;
   sourceitem->uri = uri;
@@ -1688,7 +1771,7 @@ free_source_item (GstURIDecodeBin3 * uridecodebin, GstSourceItem * item)
   if (item->handler)
     free_source_handler (uridecodebin, item->handler, TRUE);
   g_free (item->uri);
-  g_slice_free (GstSourceItem, item);
+  g_free (item);
 }
 
 static void
@@ -1705,7 +1788,7 @@ source_item_set_uri (GstSourceItem * item, const gchar * uri)
 static GstPlayItem *
 new_play_item (GstURIDecodeBin3 * dec)
 {
-  GstPlayItem *item = g_slice_new0 (GstPlayItem);
+  GstPlayItem *item = g_new0 (GstPlayItem, 1);
 
   item->uridecodebin = dec;
   item->group_id = GST_GROUP_ID_INVALID;
@@ -1722,7 +1805,7 @@ free_play_item (GstURIDecodeBin3 * dec, GstPlayItem * item)
   if (item->sub_item)
     free_source_item (dec, item->sub_item);
 
-  g_slice_free (GstPlayItem, item);
+  g_free (item);
 }
 
 static void
@@ -1832,6 +1915,7 @@ uri_src_ignore_block_probe (GstPad * pad, GstPadProbeInfo * info,
   return GST_PAD_PROBE_OK;
 }
 
+/* PLAY_ITEMS_LOCK held */
 static void
 gst_uri_decode_bin3_set_uri (GstURIDecodeBin3 * dec, const gchar * uri)
 {
@@ -1897,8 +1981,15 @@ gst_uri_decode_bin3_set_suburi (GstURIDecodeBin3 * dec, const gchar * uri)
   /* FIXME : Handle instant-uri-change. Should we just apply it automatically to
    * the current input item ? */
 
-  item = next_inactive_play_item (dec);
-  play_item_set_suburi (item, uri);
+  if (dec->input_item->posted_about_to_finish) {
+    /* WARNING : Setting sub-uri in gapless mode is unreliable */
+    GST_ELEMENT_WARNING (dec, CORE, NOT_IMPLEMENTED,
+        ("Setting sub-uri in gapless mode is not handled"),
+        ("Setting sub-uri in gapless mode is not implemented"));
+  } else {
+    item = next_inactive_play_item (dec);
+    play_item_set_suburi (item, uri);
+  }
 }
 
 /* Sync source handlers for the given play item. Might require creating/removing some
@@ -1977,6 +2068,7 @@ gst_uri_decode_bin3_change_state (GstElement * element,
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       g_atomic_int_set (&uridecodebin->shutdown, 0);
       ret = activate_play_item (uridecodebin->input_item);
+      current_updated (uridecodebin);
       if (ret == GST_STATE_CHANGE_FAILURE)
         goto failure;
       break;
@@ -2043,6 +2135,53 @@ find_source_handler_for_element (GstURIDecodeBin3 * uridecodebin,
   return NULL;
 }
 
+static GstMessage *
+gst_uri_decode_bin3_handle_redirection (GstURIDecodeBin3 * uridecodebin,
+    GstMessage * message, const GstStructure * details)
+{
+  gchar *uri = NULL;
+  GstSourceHandler *handler;
+  const gchar *location;
+  gchar *current_uri;
+
+  PLAY_ITEMS_LOCK (uridecodebin);
+  /* Find the matching handler (if any) */
+  handler = find_source_handler_for_element (uridecodebin, message->src);
+  if (!handler || !handler->play_item || !handler->play_item->main_item)
+    goto beach;
+
+  current_uri = handler->play_item->main_item->uri;
+
+  location = gst_structure_get_string ((GstStructure *) details,
+      "redirect-location");
+  GST_DEBUG_OBJECT (uridecodebin, "Handle redirection message from '%s' to '%s",
+      current_uri, location);
+
+  if (gst_uri_is_valid (location)) {
+    uri = g_strdup (location);
+  } else if (current_uri) {
+    uri = gst_uri_join_strings (current_uri, location);
+  }
+  if (!uri)
+    goto beach;
+
+  if (g_strcmp0 (current_uri, uri)) {
+    gboolean was_instant = uridecodebin->instant_uri;
+    GST_DEBUG_OBJECT (uridecodebin, "Doing instant switch to '%s'", uri);
+    uridecodebin->instant_uri = TRUE;
+    /* Force instant switch */
+    gst_uri_decode_bin3_set_uri (uridecodebin, uri);
+    uridecodebin->instant_uri = was_instant;
+    gst_message_unref (message);
+    message = NULL;
+  }
+  g_free (uri);
+
+beach:
+  PLAY_ITEMS_UNLOCK (uridecodebin);
+  return message;
+}
+
 static void
 gst_uri_decode_bin3_handle_message (GstBin * bin, GstMessage * msg)
 {
@@ -2093,6 +2232,16 @@ gst_uri_decode_bin3_handle_message (GstBin * bin, GstMessage * msg)
       break;
 
     }
+    case GST_MESSAGE_ERROR:
+    {
+      const GstStructure *details = NULL;
+
+      gst_message_parse_error_details (msg, &details);
+      if (details && gst_structure_has_field (details, "redirect-location"))
+        msg =
+            gst_uri_decode_bin3_handle_redirection (uridecodebin, msg, details);
+      break;
+    }
     default:
       break;
   }
