@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 
@@ -28,8 +28,6 @@
 
 G_END_DECLS
 
-#include "gsttcp.h"
-
 #define GST_TYPE_TCP_SERVER_SRC \
   (gst_tcp_server_src_get_type())
 #define GST_TCP_SERVER_SRC(obj) \
@@ -61,6 +59,9 @@ struct _GstTCPServerSrc {
   GCancellable *cancellable;
   GSocket *server_socket;
   GSocket *client_socket;
+
+  guint64 bytes_received;
+  GstStructure *stats;
 };
 
 struct _GstTCPServerSrcClass {
